@@ -1,7 +1,7 @@
-from __future__ import unicode_literals
+__version__ = "0.10.0"
+VERSION = tuple(__version__.split("."))
 
-VERSION = (0, 8, 7)
-__version__ = '.'.join(str(v) for v in VERSION)
+default_app_config = "mptt.apps.MpttConfig"
 
 
 def register(*args, **kwargs):
