@@ -1,7 +1,6 @@
 """
 A custom manager for working with trees of objects.
 """
-from __future__ import unicode_literals
 import functools
 import contextlib
 from itertools import groupby
@@ -10,7 +9,7 @@ from django.db import models, connections, router
 from django.db.models import F, ManyToManyField, Max, Q
 from django.utils.translation import ugettext as _
 
-from mptt.compat import remote_field
+from mptt.compat import cached_field_value
 from mptt.exceptions import CantDisableUpdates, InvalidMove
 from mptt.querysets import TreeQuerySet
 from mptt.utils import _get_tree_model
@@ -86,7 +85,7 @@ class TreeManager(models.Manager.from_queryset(TreeQuerySet)):
             self.tree_model = _get_tree_model(model)
 
             self._base_manager = None
-            if self.tree_model is not model:
+            if self.tree_model and self.tree_model is not model:
                 # _base_manager is the treemanager on tree_model
                 self._base_manager = self.tree_model._tree_manager
 
@@ -483,7 +482,7 @@ class TreeManager(models.Manager.from_queryset(TreeQuerySet)):
                     'rel_table': qn(rel_model._meta.db_table),
                     'mptt_fk': qn(rel_model._meta.get_field(rel_field).column),
                     'mptt_table': qn(self.tree_model._meta.db_table),
-                    'mptt_rel_to': qn(remote_field(mptt_field).field_name),
+                    'mptt_rel_to': qn(mptt_field.remote_field.field_name),
                     'tree_id': qn(meta.get_field(self.tree_id_attr).column),
                     'left': qn(meta.get_field(self.left_attr).column),
                     'right': qn(meta.get_field(self.right_attr).column),
@@ -493,7 +492,7 @@ class TreeManager(models.Manager.from_queryset(TreeQuerySet)):
                     'rel_table': qn(rel_model._meta.db_table),
                     'mptt_fk': qn(rel_model._meta.get_field(rel_field).column),
                     'mptt_table': qn(self.tree_model._meta.db_table),
-                    'mptt_rel_to': qn(remote_field(mptt_field).field_name),
+                    'mptt_rel_to': qn(mptt_field.remote_field.field_name),
                 }
         return queryset.extra(select={count_attr: subquery})
 
@@ -616,6 +615,7 @@ class TreeManager(models.Manager.from_queryset(TreeQuerySet)):
         move the node yourself by setting node.parent.
         """
         self._move_node(node, target, position=position)
+        node.save()
         node_moved.send(sender=node.__class__, instance=node,
                         target=target, position=position)
 
@@ -673,6 +673,79 @@ class TreeManager(models.Manager.from_queryset(TreeQuerySet)):
 
         self._rebuild_helper(pks[0], 1, tree_id)
 
+    @delegate_manager
+    def build_tree_nodes(self, data, target=None, position='last-child'):
+        """
+        Load a tree from a nested dictionary for bulk insert, returning an
+        array of records. Use to efficiently insert many nodes within a tree
+        without an expensive `rebuild`.
+
+        ::
+
+            records = MyModel.objects.build_tree_nodes({
+                'id': 7,
+                'name': 'parent',
+                'children': [
+                    {
+                        'id': 8,
+                        'parent_id': 7,
+                        'name': 'child',
+                        'children': [
+                            {
+                                'id': 9,
+                                'parent_id': 8,
+                                'name': 'grandchild',
+                            }
+                        ]
+                    }
+                ]
+            })
+            MyModel.objects.bulk_create(records)
+
+        """
+        opts = self.model._mptt_meta
+        if target:
+            tree_id = target.tree_id
+            if position in ('left', 'right'):
+                level = getattr(target, opts.level_attr)
+                if position == 'left':
+                    cursor = getattr(target, opts.left_attr)
+                else:
+                    cursor = getattr(target, opts.right_attr) + 1
+            else:
+                level = getattr(target, opts.level_attr) + 1
+                if position == 'first-child':
+                    cursor = getattr(target, opts.left_attr) + 1
+                else:
+                    cursor = getattr(target, opts.right_attr)
+        else:
+            tree_id = self._get_next_tree_id()
+            cursor = 1
+            level = 0
+
+        stack = []
+
+        def treeify(data, cursor=1, level=0):
+            data = dict(data)
+            children = data.pop('children', [])
+            node = self.model(**data)
+            stack.append(node)
+            setattr(node, opts.tree_id_attr, tree_id)
+            setattr(node, opts.level_attr, level)
+            setattr(node, opts.left_attr, cursor)
+            for child in children:
+                cursor = treeify(child, cursor=cursor + 1, level=level + 1)
+            cursor += 1
+            setattr(node, opts.right_attr, cursor)
+            return cursor
+
+        treeify(data, cursor=cursor, level=level)
+
+        if target:
+            self._create_space(2 * len(stack), cursor - 1, tree_id)
+
+        return stack
+
     def _rebuild_helper(self, pk, left, tree_id, level=0):
         opts = self.model._mptt_meta
         right = left + 1
@@ -699,17 +772,15 @@ class TreeManager(models.Manager.from_queryset(TreeQuerySet)):
 
     def _post_insert_update_cached_parent_right(self, instance, right_shift, seen=None):
         setattr(instance, self.right_attr, getattr(instance, self.right_attr) + right_shift)
-        attr = '_%s_cache' % self.parent_attr
-        if hasattr(instance, attr):
-            parent = getattr(instance, attr)
-            if parent:
-                if not seen:
-                    seen = set()
-                seen.add(instance)
-                if parent in seen:
-                    # detect infinite recursion and throw an error
-                    raise InvalidMove
-                self._post_insert_update_cached_parent_right(parent, right_shift, seen=seen)
+        parent = cached_field_value(instance, self.parent_attr)
+        if parent:
+            if not seen:
+                seen = set()
+            seen.add(instance)
+            if parent in seen:
+                # detect infinite recursion and throw an error
+                raise InvalidMove
+            self._post_insert_update_cached_parent_right(parent, right_shift, seen=seen)
 
     def _calculate_inter_tree_move_values(self, node, target, position):
         """
@@ -781,16 +852,11 @@ class TreeManager(models.Manager.from_queryset(TreeQuerySet)):
 
     def _inter_tree_move_and_close_gap(
             self, node, level_change,
-            left_right_change, new_tree_id, parent_pk=None):
+            left_right_change, new_tree_id):
         """
         Removes ``node`` from its current tree, with the given set of
         changes being applied to ``node`` and its descendants, closing
         the gap left by moving ``node`` as it does so.
-
-        If ``parent_pk`` is ``None``, this indicates that ``node`` is
-        being moved to a brand new tree as its root node, and will thus
-        have its parent field set to ``NULL``. Otherwise, ``node`` will
-        have ``parent_pk`` set for its parent field.
         """
         connection = self._get_connection(instance=node)
         qn = connection.ops.quote_name
@@ -817,20 +883,13 @@ class TreeManager(models.Manager.from_queryset(TreeQuerySet)):
                     THEN %(right)s - %%s
                 WHEN %(right)s > %%s
                     THEN %(right)s - %%s
-                ELSE %(right)s END,
-            %(parent)s = CASE
-                WHEN %(pk)s = %%s
-                    THEN %(new_parent)s
-                ELSE %(parent)s END
+                ELSE %(right)s END
         WHERE %(tree_id)s = %%s""" % {
             'table': qn(self.tree_model._meta.db_table),
             'level': qn(opts.get_field(self.level_attr).column),
             'left': qn(opts.get_field(self.left_attr).column),
             'tree_id': qn(opts.get_field(self.tree_id_attr).column),
             'right': qn(opts.get_field(self.right_attr).column),
-            'parent': qn(opts.get_field(self.parent_attr).column),
-            'pk': qn(opts.pk.column),
-            'new_parent': parent_pk is None and 'NULL' or '%s',
         }
 
         left = getattr(node, self.left_attr)
@@ -844,11 +903,8 @@ class TreeManager(models.Manager.from_queryset(TreeQuerySet)):
             gap_target_left, gap_size,
             left, right, left_right_change,
             gap_target_left, gap_size,
-            node._meta.pk.get_db_prep_value(node.pk, connection),
             getattr(node, self.tree_id_attr)
         ]
-        if parent_pk is not None:
-            params.insert(-1, parent_pk)
 
         cursor = connection.cursor()
         cursor.execute(inter_tree_move_query, params)
@@ -1037,10 +1093,8 @@ class TreeManager(models.Manager.from_queryset(TreeQuerySet)):
         # Make space for the subtree which will be moved
         self._create_space(tree_width, space_target, new_tree_id)
         # Move the subtree
-        connection = self._get_connection(instance=node)
         self._inter_tree_move_and_close_gap(
-            node, level_change, left_right_change, new_tree_id,
-            parent._meta.pk.get_db_prep_value(parent.pk, connection))
+            node, level_change, left_right_change, new_tree_id)
 
         # Update the node to be consistent with the updated
         # tree in the database.
@@ -1146,18 +1200,12 @@ class TreeManager(models.Manager.from_queryset(TreeQuerySet)):
                   THEN %(right)s + %%s
                 WHEN %(right)s >= %%s AND %(right)s <= %%s
                   THEN %(right)s + %%s
-                ELSE %(right)s END,
-            %(parent)s = CASE
-                WHEN %(pk)s = %%s
-                  THEN %%s
-                ELSE %(parent)s END
+                ELSE %(right)s END
         WHERE %(tree_id)s = %%s""" % {
             'table': qn(self.tree_model._meta.db_table),
             'level': qn(opts.get_field(self.level_attr).column),
             'left': qn(opts.get_field(self.left_attr).column),
             'right': qn(opts.get_field(self.right_attr).column),
-            'parent': qn(opts.get_field(self.parent_attr).column),
-            'pk': qn(opts.pk.column),
             'tree_id': qn(opts.get_field(self.tree_id_attr).column),
         }
 
@@ -1168,8 +1216,6 @@ class TreeManager(models.Manager.from_queryset(TreeQuerySet)):
             left_boundary, right_boundary, gap_size,
             left, right, left_right_change,
             left_boundary, right_boundary, gap_size,
-            node._meta.get_field(node._meta.pk.name).get_db_prep_value(node.pk, connection),
-            parent._meta.get_field(parent._meta.pk.name).get_db_prep_value(parent.pk, connection),
             tree_id])
 
         # Update the node to be consistent with the updated
@@ -1217,11 +1263,7 @@ class TreeManager(models.Manager.from_queryset(TreeQuerySet)):
         SET %(level)s = %(level)s - %%s,
             %(left)s = %(left)s - %%s,
             %(right)s = %(right)s - %%s,
-            %(tree_id)s = %%s,
-            %(parent)s = CASE
-                WHEN %(pk)s = %%s
-                    THEN %%s
-                ELSE %(parent)s END
+            %(tree_id)s = %%s
         WHERE %(left)s >= %%s AND %(left)s <= %%s
           AND %(tree_id)s = %%s""" % {
             'table': qn(self.tree_model._meta.db_table),
@@ -1229,16 +1271,12 @@ class TreeManager(models.Manager.from_queryset(TreeQuerySet)):
             'left': qn(opts.get_field(self.left_attr).column),
             'right': qn(opts.get_field(self.right_attr).column),
             'tree_id': qn(opts.get_field(self.tree_id_attr).column),
-            'parent': qn(opts.get_field(self.parent_attr).column),
-            'pk': qn(opts.pk.column),
         }
 
         cursor = connection.cursor()
         cursor.execute(move_tree_query, [
             level_change, left_right_change, left_right_change,
             new_tree_id,
-            node._meta.pk.get_db_prep_value(node.pk, connection),
-            parent._meta.pk.get_db_prep_value(parent.pk, connection),
             left, right, tree_id])
 
         # Update the former root node to be consistent with the updated
