@@ -1,6 +1,4 @@
-from __future__ import unicode_literals
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 from uuid import uuid4
 
 import mptt
@@ -18,20 +16,13 @@ class CustomTreeQueryset(QuerySet):
 
 class CustomTreeManager(TreeManager):
 
-    def get_query_set(self):
-        return CustomTreeQueryset(model=self.model, using=self._db)
-
     def get_queryset(self):
-        # Django 1.8 removed the fallbacks here.
         return CustomTreeQueryset(model=self.model, using=self._db)
 
-    def get_empty_query_set(self):
-        return self.get_queryset().none()
 
-
-@python_2_unicode_compatible
 class Category(MPTTModel):
     name = models.CharField(max_length=50)
+    visible = models.BooleanField(default=True)
     parent = TreeForeignKey(
         'self', null=True, blank=True, related_name='children',
         on_delete=models.CASCADE)
@@ -45,7 +36,6 @@ class Category(MPTTModel):
     delete.alters_data = True
 
 
-@python_2_unicode_compatible
 class Item(models.Model):
 
     name = models.CharField(max_length=100)
@@ -60,7 +50,6 @@ class Item(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Genre(MPTTModel):
     name = models.CharField(max_length=50, unique=True)
     parent = TreeForeignKey(
@@ -86,7 +75,6 @@ class Insert(MPTTModel):
         on_delete=models.CASCADE)
 
 
-@python_2_unicode_compatible
 class MultiOrder(MPTTModel):
     name = models.CharField(max_length=50)
     size = models.PositiveIntegerField()
@@ -117,7 +105,9 @@ class Node(MPTTModel):
 
 
 class UUIDNode(MPTTModel):
-    parent = models.ForeignKey('self', null=True, blank=True, related_name='children')
+    parent = models.ForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
     uuid = models.UUIDField(primary_key=True, default=uuid4)
     name = models.CharField(max_length=50)
 
@@ -125,7 +115,6 @@ class UUIDNode(MPTTModel):
         return self.name
 
 
-@python_2_unicode_compatible
 class OrderedInsertion(MPTTModel):
     name = models.CharField(max_length=50)
     parent = TreeForeignKey(
@@ -150,11 +139,10 @@ class NewStyleMPTTMeta(MPTTModel):
         'self', null=True, blank=True, related_name='children',
         on_delete=models.CASCADE)
 
-    class MPTTMeta(object):
+    class MPTTMeta:
         left_attr = 'testing'
 
 
-@python_2_unicode_compatible
 class Person(MPTTModel):
     name = models.CharField(max_length=50)
     parent = TreeForeignKey(
@@ -172,7 +160,6 @@ class Student(Person):
     type = models.CharField(max_length=50)
 
 
-@python_2_unicode_compatible
 class CustomPKName(MPTTModel):
     my_id = models.AutoField(db_column='my_custom_name', primary_key=True)
     name = models.CharField(max_length=50)
@@ -283,6 +270,22 @@ class SwappedInModel(MPTTModel):
     name = models.CharField(max_length=50)
 
 
+# Default manager
+class MultipleManager(TreeManager):
+    def get_queryset(self):
+        return super(MultipleManager, self).get_queryset().exclude(published=False)
+
+
+class MultipleManagerModel(MPTTModel):
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
+    published = models.BooleanField()
+
+    objects = TreeManager()
+    foo_objects = MultipleManager()
+
+
 class AutoNowDateFieldModel(MPTTModel):
     parent = TreeForeignKey(
         'self', null=True, blank=True, related_name='children',
@@ -297,6 +300,7 @@ class AutoNowDateFieldModel(MPTTModel):
 class Group(models.Model):
     name = models.CharField(max_length=100)
 
+
 TreeForeignKey(
     Group, blank=True, null=True, on_delete=models.CASCADE
 ).contribute_to_class(Group, 'parent')
@@ -309,5 +313,40 @@ class Book(MPTTModel):
         on_delete=models.CASCADE)
 
     name = models.CharField(max_length=50)
-    fk = TreeForeignKey(Category, null=True, blank=True, related_name='books_fk')
-    m2m = TreeManyToManyField(Category, null=True, blank=True, related_name='books_m2m')
+    fk = TreeForeignKey(
+        Category, null=True, blank=True, related_name='books_fk',
+        on_delete=models.CASCADE)
+    m2m = TreeManyToManyField(Category, blank=True, related_name='books_m2m')
+
+
+class UniqueTogetherModel(MPTTModel):
+    class Meta:
+        unique_together = (('parent','code',),)
+    parent = TreeForeignKey('self', null=True, on_delete=models.CASCADE)
+    code = models.CharField(max_length=10)
+
+
+class NullableOrderedInsertionModel(MPTTModel):
+    name = models.CharField(max_length=50, null=True)
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
+
+    class MPTTMeta:
+        order_insertion_by = ['name']
+
+    def __str__(self):
+        return self.name
+
+
+class NullableDescOrderedInsertionModel(MPTTModel):
+    name = models.CharField(max_length=50, null=True)
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
+
+    class MPTTMeta:
+        order_insertion_by = ['-name']
+
+    def __str__(self):
+        return self.name
