@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 import io
 import os
 import re
@@ -7,15 +5,12 @@ import sys
 import tempfile
 import unittest
 
-from django import forms
 from django.contrib.auth.models import Group, User
 from django.db.models import Q
 from django.db.models.query_utils import DeferredAttribute
 from django.apps import apps
-from django.forms.models import modelform_factory
 from django.template import Template, TemplateSyntaxError, Context
 from django.test import RequestFactory, TestCase
-from django.utils.six import string_types, PY3, b, assertRaisesRegex
 from django.contrib.admin.views.main import ChangeList
 from django.contrib.admin import ModelAdmin, site
 from mptt.admin import TreeRelatedFieldListFilter
@@ -25,11 +20,7 @@ try:
 except ImportError:
     mock_signal_receiver = None
 
-from mptt.admin import JS
 from mptt.exceptions import CantDisableUpdates, InvalidMove
-from mptt.forms import (
-    MPTTAdminForm, TreeNodeChoiceField, TreeNodeMultipleChoiceField,
-    MoveNodeForm)
 from mptt.models import MPTTModel
 from mptt.managers import TreeManager
 from mptt.signals import node_moved
@@ -39,8 +30,8 @@ from mptt.utils import print_debug_info
 from myapp.models import (
     Category, Item, Genre, CustomPKName, SingleProxyModel, DoubleProxyModel,
     ConcreteModel, OrderedInsertion, AutoNowDateFieldModel, Person,
-    CustomTreeQueryset, Node, ReferencingModel, CustomTreeManager, Book,
-    UUIDNode, Student)
+    CustomTreeQueryset, Node, CustomTreeManager, Book, UUIDNode, Student,
+    MultipleManagerModel, UniqueTogetherModel, NullableOrderedInsertionModel, NullableDescOrderedInsertionModel)
 
 
 def get_tree_details(nodes):
@@ -59,6 +50,7 @@ def get_tree_details(nodes):
                        getattr(n, opts.left_attr), getattr(n, opts.right_attr))
                       for n in nodes])
 
+
 leading_whitespace_re = re.compile(r'^\s+', re.MULTILINE)
 
 
@@ -75,10 +67,10 @@ def tree_details(text):
 class TreeTestCase(TestCase):
 
     def assertTreeEqual(self, tree1, tree2):
-        if not isinstance(tree1, string_types):
+        if not isinstance(tree1, str):
             tree1 = get_tree_details(tree1)
         tree1 = tree_details(tree1)
-        if not isinstance(tree2, string_types):
+        if not isinstance(tree2, str):
             tree2 = get_tree_details(tree2)
         tree2 = tree_details(tree2)
         return self.assertEqual(tree1, tree2, "\n%r\n != \n%r" % (tree1, tree2))
@@ -87,6 +79,8 @@ class TreeTestCase(TestCase):
 class DocTestTestCase(TreeTestCase):
 
     def test_run_doctest(self):
+        import doctest
+
         class DummyStream:
             content = ""
             encoding = 'utf8'
@@ -101,33 +95,17 @@ class DocTestTestCase(TreeTestCase):
         before = sys.stdout
         sys.stdout = dummy_stream
 
-        with open(os.path.join(os.path.dirname(__file__), 'doctests.txt')) as f:
-            with tempfile.NamedTemporaryFile() as temp:
-                text = f.read()
-
-                if PY3:
-                    # unicode literals in the doctests screw up doctest on py3.
-                    # this is pretty icky, but I can't find any other
-                    # workarounds :(
-                    text = re.sub(r"""\bu(["\'])""", r"\1", text)
-                    temp.write(b(text))
-                else:
-                    temp.write(text)
-
-                temp.flush()
-
-                import doctest
-                doctest.testfile(
-                    temp.name,
-                    module_relative=False,
-                    optionflags=doctest.IGNORE_EXCEPTION_DETAIL | doctest.ELLIPSIS,
-                    encoding='utf-8',
-                )
-                sys.stdout = before
-                content = dummy_stream.content
-                if content:
-                    before.write(content + '\n')
-                    self.fail()
+        doctest.testfile(
+            os.path.join(os.path.dirname(__file__), 'doctests.txt'),
+            module_relative=False,
+            optionflags=doctest.IGNORE_EXCEPTION_DETAIL | doctest.ELLIPSIS,
+            encoding='utf-8',
+        )
+        sys.stdout = before
+        content = dummy_stream.content
+        if content:
+            before.write(content + '\n')
+            self.fail()
 
 # genres.json defines the following tree structure
 #
@@ -532,6 +510,22 @@ class DeletionTestCase(TreeTestCase):
             10 8 1 2 11 12
         """)
 
+    def test_delete_multiple_nodes(self):
+        """Regression test for Issue 576."""
+        queryset = Category.objects.filter(id__in=[6,7])
+        for category in queryset:
+            category .delete()
+
+        self.assertTreeEqual(Category.objects.all(), """
+            1 - 1 0 1 16
+            2 1 1 1 2 7
+            3 2 1 2 3 4
+            4 2 1 2 5 6
+            5 1 1 1 8 9
+            8 1 1 1 10 15
+            9 8 1 2 11 12
+            10 8 1 2 13 14""")
+
 
 class IntraTreeMovementTestCase(TreeTestCase):
     pass
@@ -961,8 +955,39 @@ class DelayedUpdatesTestCase(TreeTestCase):
         """)
 
 
-class OrderedInsertionDelayedUpdatesTestCase(TreeTestCase):
+class OrderedInsertionSortingTestCase(TestCase):
+    def test_insert_unordered_stuff(self):
+        root = OrderedInsertion.objects.create(name="")
 
+        # "b" gets inserted first,
+        b = OrderedInsertion.objects.create(name="b", parent=root)
+
+        # "a" gets inserted later,
+        a = OrderedInsertion.objects.create(name="a", parent=root)
+        # ... but specifying OrderedInsertion.MPTTMeta.order_insertion_by
+        # tells django-mptt to order added items by the name. So basically
+        # instance "a", added later, will get the first place in the
+        # tree. So what's exactly seems to be the problem?
+        #
+        # The problem is, item "b" will not get refreshed in any
+        # way. We need to reload it manually or else there will be problems
+        # like the one demonstrated below:
+
+        self.assertIn(a, a.get_ancestors(include_self=True))
+
+        # This will raise an AssertionError, unless we reload the item from
+        # the database. As long as we won't come up with a sensible way
+        # of reloading all Django instances pointing to a given row in the
+        # database...
+        #   self.assertIn(b, b.get_ancestors(include_self=True)))
+        self.assertRaises(AssertionError, self.assertIn, b, b.get_ancestors(include_self=True))
+
+        # ... we need to reload it properly ourselves:
+        b.refresh_from_db()
+        self.assertIn(b, b.get_ancestors(include_self=True))
+
+
+class OrderedInsertionDelayedUpdatesTestCase(TreeTestCase):
     def setUp(self):
         self.c = OrderedInsertion.objects.create(name="c")
         self.d = OrderedInsertion.objects.create(name="d", parent=self.c)
@@ -1299,6 +1324,13 @@ class ManagerTests(TreeTestCase):
                 Category.objects.all(), include_self=True)
             self.assertEqual(len(qs), 10)
 
+    def test_default_manager_with_multiple_managers(self):
+        """
+        Test that a model with multiple managers defined always uses the
+        default manager as the tree manager.
+        """
+        self.assertEqual(type(MultipleManagerModel._tree_manager), TreeManager)
+
 
 class CacheTreeChildrenTestCase(TreeTestCase):
 
@@ -1547,82 +1579,6 @@ class RegisteredRemoteModel(TreeTestCase):
         g1.save()
 
 
-class TestForms(TreeTestCase):
-    fixtures = ['categories.json']
-
-    def test_adminform_instantiation(self):
-        # https://github.com/django-mptt/django-mptt/issues/264
-        c = Category.objects.get(name='Nintendo Wii')
-        CategoryForm = modelform_factory(
-            Category,
-            form=MPTTAdminForm,
-            fields=('name', 'parent'),
-        )
-        self.assertTrue(CategoryForm(instance=c))
-
-        # Test that the parent field is properly limited. (queryset)
-        form = CategoryForm({
-            'name': c.name,
-            'parent': c.children.all()[0].pk,
-        }, instance=c)
-        self.assertFalse(form.is_valid())
-        self.assertIn(
-            'Select a valid choice',
-            '%s' % form.errors)
-
-        # Test that even though we remove the field queryset limit,
-        # validation still fails.
-        form = CategoryForm({
-            'name': c.name,
-            'parent': c.children.all()[0].pk,
-        }, instance=c)
-        form.fields['parent'].queryset = Category.objects.all()
-        self.assertFalse(form.is_valid())
-        self.assertIn(
-            'Invalid parent',
-            '%s' % form.errors)
-
-    def test_field_types(self):
-        ReferencingModelForm = modelform_factory(
-            ReferencingModel,
-            exclude=('id',))
-
-        form = ReferencingModelForm()
-
-        # Also check whether we have the correct form field type
-        self.assertTrue(isinstance(
-            form.fields['fk'],
-            TreeNodeChoiceField))
-        self.assertTrue(isinstance(
-            form.fields['one'],
-            TreeNodeChoiceField))
-        self.assertTrue(isinstance(
-            form.fields['m2m'],
-            TreeNodeMultipleChoiceField))
-
-    def test_movenodeform(self):
-        c = Category.objects.get(pk=2)
-        form = MoveNodeForm(c, {
-            'target': '5',
-            'position': 'first-child',
-        })
-        self.assertTrue(form.is_valid())
-        form.save()
-
-        self.assertTreeEqual(Category.objects.all(), '''
-            1 - 1 0 1 20
-            5 1 1 1 2 13
-            2 5 1 2 3 8
-            3 2 1 3 4 5
-            4 2 1 3 6 7
-            6 5 1 2 9 10
-            7 5 1 2 11 12
-            8 1 1 1 14 19
-            9 8 1 2 15 16
-            10 8 1 2 17 18
-        ''')
-
-
 class TestAltersData(TreeTestCase):
 
     def test_alters_data(self):
@@ -1646,14 +1602,22 @@ class TestAltersData(TreeTestCase):
 class TestDebugInfo(TreeTestCase):
     fixtures = ['categories.json']
 
-    def test_debug_info(self):  # Currently fails either on PY2 or PY3.
-        stream_type = io.StringIO if PY3 else io.BytesIO
-        with stream_type() as out:
+    def test_debug_info(self):
+        with io.StringIO() as out:
             print_debug_info(Category.objects.all(), file=out)
             output = out.getvalue()
 
         self.assertIn('1,0,,1,1,20', output)
 
+    def test_debug_info_with_non_ascii_representations(self):
+        Category.objects.create(name='El niño')
+
+        with io.StringIO() as out:
+            print_debug_info(Category.objects.all(), file=out)
+            output = out.getvalue()
+
+        self.assertIn('El niño', output)
+
 
 class AdminBatch(TreeTestCase):
     fixtures = ['categories.json']
@@ -1670,7 +1634,7 @@ class AdminBatch(TreeTestCase):
             10)
 
         mptt_opts = Category._mptt_meta
-        self.assertEqual(
+        self.assertSequenceEqual(
             response.context['cl'].result_list.query.order_by[:2],
             [mptt_opts.tree_id_attr, mptt_opts.left_attr])
 
@@ -1714,8 +1678,7 @@ class TestUnsaved(TreeTestCase):
             'get_root',
             'get_siblings',
         ]:
-            assertRaisesRegex(
-                self,
+            self.assertRaisesRegex(
                 ValueError,
                 'Cannot call %s on unsaved Genre instances' % method,
                 getattr(Genre(), method))
@@ -1882,6 +1845,17 @@ class CacheChildrenTestCase(TreeTestCase):
                     for child2 in child.get_children():
                         self.assertIsInstance(child2, Genre)
 
+    def test_hide_nodes(self):
+        """
+        Test that caching a tree with missing nodes works
+        """
+        root = Category.objects.create(name='Root', visible=False)
+        child = Category.objects.create(name='Child', parent=root)
+        root2 = Category.objects.create(name='Root2')
+
+        list(Category.objects.all().get_cached_trees()) == [root, child, root2]
+        list(Category.objects.filter(visible=True).get_cached_trees()) == [child, root2]
+
 
 @unittest.skipUnless(mock_signal_receiver, "Signals tests require mock_django installed")
 class Signals(TestCase):
@@ -2066,23 +2040,6 @@ class DraggableMPTTAdminTestCase(TreeTestCase):
             3 2 2 1 2 3
             """)
 
-    def test_js(self):
-        media = forms.Media()
-        media.add_js([
-            JS('asset1.js', {}),
-            JS('asset2.js', {'id': 'something', 'answer': '"42"'}),
-        ])
-
-        # We can test the exact representation since forms.Media has been
-        # really stable for a long time, and JS() uses flatatt which
-        # alphabetically sorts its attributes.
-        self.assertEqual(
-            '%s' % media,
-            '<script type="text/javascript" src="/static/asset1.js"></script>\n'
-            '<script type="text/javascript" src="/static/asset2.js"'
-            ' answer="&quot;42&quot;" id="something"></script>'
-        )
-
 
 class BookAdmin(ModelAdmin):
     list_filter = (
@@ -2103,6 +2060,7 @@ class CategoryAdmin(ModelAdmin):
 class ListFiltersTests(TestCase):
 
     def setUp(self):
+        self.user = User.objects.create_superuser('admin', 'test@example.com', 'p')
         self.request_factory = RequestFactory()
 
         self.parent_category = Category.objects.create(name='Parent category')
@@ -2121,19 +2079,28 @@ class ListFiltersTests(TestCase):
         self.book2.m2m.add(self.parent_category)
         self.book3.m2m.add(self.simple_category)
 
+    def get_request(self, path, params=None):
+        req = self.request_factory.get(path, params)
+        req.user = self.user
+        return req
+
     def get_changelist(self, request, model, modeladmin):
-        return ChangeList(
+        args = [
             request, model, modeladmin.list_display,
             modeladmin.list_display_links, modeladmin.list_filter,
             modeladmin.date_hierarchy, modeladmin.search_fields,
             modeladmin.list_select_related, modeladmin.list_per_page,
             modeladmin.list_max_show_all, modeladmin.list_editable, modeladmin,
-        )
+        ]
+        if hasattr(modeladmin, 'sortable_by'):
+            # New in Django 2.1
+            args.append(modeladmin.sortable_by)
+        return ChangeList(*args)
 
     def test_treerelatedfieldlistfilter_foreignkey(self):
         modeladmin = BookAdmin(Book, site)
 
-        request = self.request_factory.get('/')
+        request = self.get_request('/')
         changelist = self.get_changelist(request, Book, modeladmin)
 
         # Make sure that all categories are present in the referencing model's list filter
@@ -2146,7 +2113,7 @@ class ListFiltersTests(TestCase):
         ]
         self.assertEqual(sorted(filterspec.lookup_choices), sorted(expected))
 
-        request = self.request_factory.get('/', {'fk__isnull': 'True'})
+        request = self.get_request('/', {'fk__isnull': 'True'})
         changelist = self.get_changelist(request, Book, modeladmin)
         queryset = changelist.get_queryset(request)
         self.assertEqual(list(queryset), [self.book4])
@@ -2158,25 +2125,25 @@ class ListFiltersTests(TestCase):
         self.assertEqual(choices[-1]['query_string'], '?fk__isnull=True')
 
         # Make sure child's categories books included
-        request = self.request_factory.get('/', {'fk__id__inhierarchy': self.parent_category.pk})
+        request = self.get_request('/', {'fk__id__inhierarchy': self.parent_category.pk})
         changelist = self.get_changelist(request, Book, modeladmin)
         queryset = changelist.get_queryset(request)
         self.assertEqual((list(queryset)), [self.book1, self.book2])
 
         # Make sure filter for child category works as expected
-        request = self.request_factory.get('/', {'fk__id__inhierarchy': self.child_category1.pk})
+        request = self.get_request('/', {'fk__id__inhierarchy': self.child_category1.pk})
         changelist = self.get_changelist(request, Book, modeladmin)
         queryset = changelist.get_queryset(request)
         self.assertEqual((list(queryset)), [self.book1])
 
         # Make sure filter for empty category works as expected
-        request = self.request_factory.get('/', {'fk__id__inhierarchy': self.child_category2.pk})
+        request = self.get_request('/', {'fk__id__inhierarchy': self.child_category2.pk})
         changelist = self.get_changelist(request, Book, modeladmin)
         queryset = changelist.get_queryset(request)
         self.assertEqual(queryset.count(), 0)
 
         # Make sure filter for simple category with no hierarchy works as expected
-        request = self.request_factory.get('/', {'fk__id__inhierarchy': self.simple_category.pk})
+        request = self.get_request('/', {'fk__id__inhierarchy': self.simple_category.pk})
         changelist = self.get_changelist(request, Book, modeladmin)
         queryset = changelist.get_queryset(request)
         self.assertEqual((list(queryset)), [self.book3])
@@ -2184,7 +2151,7 @@ class ListFiltersTests(TestCase):
     def test_treerelatedfieldlistfilter_manytomany(self):
         modeladmin = BookAdmin(Book, site)
 
-        request = self.request_factory.get('/')
+        request = self.get_request('/')
         changelist = self.get_changelist(request, Book, modeladmin)
 
         # Make sure that all categories are present in the referencing model's list filter
@@ -2198,7 +2165,7 @@ class ListFiltersTests(TestCase):
         ]
         self.assertEqual(sorted(filterspec.lookup_choices), sorted(expected))
 
-        request = self.request_factory.get('/', {'m2m__isnull': 'True'})
+        request = self.get_request('/', {'m2m__isnull': 'True'})
         changelist = self.get_changelist(request, Book, modeladmin)
 
         # Make sure the correct queryset is returned
@@ -2212,25 +2179,25 @@ class ListFiltersTests(TestCase):
         self.assertEqual(choices[-1]['query_string'], '?m2m__isnull=True')
 
         # Make sure child's categories books included
-        request = self.request_factory.get('/', {'m2m__id__inhierarchy': self.parent_category.pk})
+        request = self.get_request('/', {'m2m__id__inhierarchy': self.parent_category.pk})
         changelist = self.get_changelist(request, Book, modeladmin)
         queryset = changelist.get_queryset(request)
         self.assertEqual((list(queryset)), [self.book1, self.book2])
 
         # Make sure filter for child category works as expected
-        request = self.request_factory.get('/', {'m2m__id__inhierarchy': self.child_category1.pk})
+        request = self.get_request('/', {'m2m__id__inhierarchy': self.child_category1.pk})
         changelist = self.get_changelist(request, Book, modeladmin)
         queryset = changelist.get_queryset(request)
         self.assertEqual((list(queryset)), [self.book1])
 
         # Make sure filter for empty category works as expected
-        request = self.request_factory.get('/', {'fk__id__inhierarchy': self.child_category2.pk})
+        request = self.get_request('/', {'fk__id__inhierarchy': self.child_category2.pk})
         changelist = self.get_changelist(request, Book, modeladmin)
         queryset = changelist.get_queryset(request)
         self.assertEqual(queryset.count(), 0)
 
         # Make sure filter for simple category with no hierarchy works as expected
-        request = self.request_factory.get('/', {'m2m__id__inhierarchy': self.simple_category.pk})
+        request = self.get_request('/', {'m2m__id__inhierarchy': self.simple_category.pk})
         changelist = self.get_changelist(request, Book, modeladmin)
         queryset = changelist.get_queryset(request)
         self.assertEqual((list(queryset)), [self.book3])
@@ -2239,7 +2206,7 @@ class ListFiltersTests(TestCase):
         modeladmin = CategoryAdmin(Category, site)
 
         # FK relationship -----
-        request = self.request_factory.get('/', {'books_fk__isnull': 'True'})
+        request = self.get_request('/', {'books_fk__isnull': 'True'})
         changelist = self.get_changelist(request, Category, modeladmin)
 
         # Make sure the correct queryset is returned
@@ -2253,26 +2220,26 @@ class ListFiltersTests(TestCase):
         self.assertEqual(choices[-1]['query_string'], '?books_fk__isnull=True')
 
         # Make sure child's books categories included
-        request = self.request_factory.get('/', {'books_fk__id__inhierarchy': self.book1.pk})
+        request = self.get_request('/', {'books_fk__id__inhierarchy': self.book1.pk})
         changelist = self.get_changelist(request, Category, modeladmin)
         queryset = changelist.get_queryset(request)
         self.assertEqual((list(queryset)),
                          [self.parent_category, self.child_category1, self.simple_category])
 
         # Make sure filter for child book works as expected
-        request = self.request_factory.get('/', {'books_fk__id__inhierarchy': self.book2.pk})
+        request = self.get_request('/', {'books_fk__id__inhierarchy': self.book2.pk})
         changelist = self.get_changelist(request, Category, modeladmin)
         queryset = changelist.get_queryset(request)
         self.assertEqual((list(queryset)), [self.parent_category])
 
         # Make sure filter for book with no category works as expected
-        request = self.request_factory.get('/', {'books_fk__id__inhierarchy': self.book4.pk})
+        request = self.get_request('/', {'books_fk__id__inhierarchy': self.book4.pk})
         changelist = self.get_changelist(request, Category, modeladmin)
         queryset = changelist.get_queryset(request)
         self.assertEqual(queryset.count(), 0)
 
         # M2M relationship -----
-        request = self.request_factory.get('/', {'books_m2m__isnull': 'True'})
+        request = self.get_request('/', {'books_m2m__isnull': 'True'})
         changelist = self.get_changelist(request, Category, modeladmin)
         queryset = changelist.get_queryset(request)
         self.assertEqual(list(queryset), [self.child_category2])
@@ -2284,20 +2251,20 @@ class ListFiltersTests(TestCase):
         self.assertEqual(choices[-1]['query_string'], '?books_m2m__isnull=True')
 
         # Make sure child's books categories included
-        request = self.request_factory.get('/', {'books_m2m__id__inhierarchy': self.book1.pk})
+        request = self.get_request('/', {'books_m2m__id__inhierarchy': self.book1.pk})
         changelist = self.get_changelist(request, Category, modeladmin)
         queryset = changelist.get_queryset(request)
         self.assertEqual((list(queryset)),
                          [self.parent_category, self.child_category1, self.simple_category])
 
         # Make sure filter for child book works as expected
-        request = self.request_factory.get('/', {'books_m2m__id__inhierarchy': self.book2.pk})
+        request = self.get_request('/', {'books_m2m__id__inhierarchy': self.book2.pk})
         changelist = self.get_changelist(request, Category, modeladmin)
         queryset = changelist.get_queryset(request)
         self.assertEqual((list(queryset)), [self.parent_category])
 
         # Make sure filter for book with no category works as expected
-        request = self.request_factory.get('/', {'books_m2m__id__inhierarchy': self.book4.pk})
+        request = self.get_request('/', {'books_m2m__id__inhierarchy': self.book4.pk})
         changelist = self.get_changelist(request, Category, modeladmin)
         queryset = changelist.get_queryset(request)
         self.assertEqual(queryset.count(), 0)
@@ -2351,3 +2318,198 @@ class DirectParentAssignment(TreeTestCase):
         n2 = Node.objects.create()
         n1.parent_id = n2.id
         n1.save()
+
+
+class MovingNodeWithUniqueConstraint(TreeTestCase):
+    def test_unique_together_move_to_same_parent_change_code(self):
+        """Regression test for #466 1"""
+
+        UniqueTogetherModel.objects.all().delete()
+
+        a = UniqueTogetherModel.objects.create(code='a', parent=None)
+        b = UniqueTogetherModel.objects.create(code='b', parent=None)
+        a1 = UniqueTogetherModel.objects.create(code='1', parent=a)
+        b1 = UniqueTogetherModel.objects.create(code='1', parent=b)
+        b1.parent, b1.code = a, '2'  # b1 -> a2
+        b1.save()
+
+        self.assertTreeEqual(UniqueTogetherModel.objects.all(), """
+            1 - 1 0 1 6
+            3 1 1 1 2 3
+            4 1 1 1 4 5
+            2 - 2 0 1 2
+        """)
+
+    def test_unique_together_move_to_same_code_change_parent(self):
+        """Regression test for #466 1"""
+
+        UniqueTogetherModel.objects.all().delete()
+
+        a = UniqueTogetherModel.objects.create(code='a', parent=None)
+        b = UniqueTogetherModel.objects.create(code='b', parent=None)
+        a1 = UniqueTogetherModel.objects.create(code='1', parent=a)
+        a2 = UniqueTogetherModel.objects.create(code='2', parent=a)
+        a2.parent, a2.code = b, '1'  # a2 -> b1
+        a2.save()
+
+        self.assertTreeEqual(UniqueTogetherModel.objects.all(), """
+            1 - 1 0 1 4
+            3 1 1 1 2 3
+            2 - 2 0 1 4
+            4 2 2 1 2 3
+        """)
+
+
+class NullableOrderedInsertion(TreeTestCase):
+    def test_nullable_ordered_insertion(self):
+
+        genreA = NullableOrderedInsertionModel.objects.create(name='A', parent=None)
+        genreA1 = NullableOrderedInsertionModel.objects.create(name='A1', parent=genreA)
+        genreAnone = NullableOrderedInsertionModel.objects.create(name=None, parent=genreA)
+
+        self.assertTreeEqual(NullableOrderedInsertionModel.objects.all(), """
+            1 - 1 0 1 6
+            3 1 1 1 2 3
+            2 1 1 1 4 5
+        """)
+
+    def test_nullable_ordered_insertion_desc(self):
+
+        genreA = NullableDescOrderedInsertionModel.objects.create(name='A', parent=None)
+        genreA1 = NullableDescOrderedInsertionModel.objects.create(name='A1', parent=genreA)
+        genreAnone = NullableDescOrderedInsertionModel.objects.create(name=None, parent=genreA)
+
+        self.assertTreeEqual(NullableDescOrderedInsertionModel.objects.all(), """
+            1 - 1 0 1 6
+            2 1 1 1 2 3
+            3 1 1 1 4 5
+        """)
+
+
+class ModelMetaIndexes(TreeTestCase):
+    def test_no_index_set(self):
+        class SomeModel(MPTTModel):
+            class Meta:
+                app_label = 'myapp'
+
+        tree_id_attr = getattr(SomeModel._mptt_meta, 'tree_id_attr')
+        self.assertTrue(SomeModel._meta.get_field(tree_id_attr).db_index)
+
+        for key in ('right_attr', 'left_attr', 'level_attr'):
+            field_name = getattr(SomeModel._mptt_meta, key)
+            self.assertFalse(SomeModel._meta.get_field(field_name).db_index)
+
+    def test_index_together(self):
+        already_idx = [['tree_id', 'lft'], ('tree_id', 'lft')]
+        no_idx = [tuple(), list()]
+        some_idx = [['tree_id'], ('tree_id',), [['tree_id']], (('tree_id',),)]
+
+        for idx, case in enumerate(already_idx + no_idx + some_idx):
+            class Meta:
+                index_together = case
+                app_label = 'myapp'
+
+            # Use type() here and in test_index_together_different_attr over
+            # an explicit class X(MPTTModel):, as this throws a warning that
+            # re-registering models with the same name (which is what an explicit
+            # class does) could cause errors. Kind of... weird, but surprisingly
+            # effective.
+
+            SomeModel = type(str('model_{0}'.format(idx)), (MPTTModel,), {
+                'Meta': Meta,
+                '__module__': __name__,
+            })
+
+            self.assertIn(('tree_id', 'lft'), SomeModel._meta.index_together)
+
+    def test_index_together_different_attr(self):
+        already_idx = [['abc', 'def'], ('abc', 'def')]
+        no_idx = [tuple(), list()]
+        some_idx = [['abc'], ('abc',), [['abc']], (('abc',),)]
+
+        for idx, case in enumerate(already_idx + no_idx + some_idx):
+            class MPTTMeta:
+                tree_id_attr = 'abc'
+                left_attr = 'def'
+
+            class Meta:
+                index_together = case
+                app_label = 'myapp'
+
+            SomeModel = type(str('model__different_attr_{0}'.format(idx)), (MPTTModel,), {
+                'MPTTMeta': MPTTMeta,
+                'Meta': Meta,
+                '__module__': str(__name__)
+            })
+
+            self.assertIn(('abc', 'def'), SomeModel._meta.index_together)
+
+
+class BulkLoadTests(TestCase):
+
+    fixtures = ['categories.json']
+
+    def setUp(self):
+        self.games = {
+            'id': 11,
+            'name': 'Role-playing',
+            'children': [
+                {
+                    'id': 12,
+                    'parent_id': 11,
+                    'name': 'Single-player',
+                },
+                {
+                    'id': 13,
+                    'parent_id': 11,
+                    'name': 'Multi-player',
+                },
+            ],
+        }
+
+    def test_bulk_root(self):
+        data = {
+            'id': 11,
+            'name': 'Enterprise Software',
+            'children': [
+                {
+                    'id': 12,
+                    'parent_id': 11,
+                    'name': 'Databases',
+                },
+                {
+                    'id': 13,
+                    'parent_id': 11,
+                    'name': 'Timekeeping',
+                },
+            ],
+        }
+        records = Category.objects.build_tree_nodes(data)
+        self.assertEqual(len(records), 3)
+        self.assertEqual((records[0].lft, records[0].rght), (1, 6))
+        self.assertEqual((records[1].lft, records[1].rght), (2, 3))
+        self.assertEqual((records[2].lft, records[2].rght), (4, 5))
+
+    def test_bulk_last_child(self):
+        games = Category.objects.get(id=3)
+        records = Category.objects.build_tree_nodes(self.games, target=games)
+        self.assertEqual(len(records), 3)
+        for record in records:
+            self.assertEqual(record.tree_id, games.tree_id)
+        self.assertEqual((records[0].lft, records[0].rght), (4, 9))
+        self.assertEqual((records[1].lft, records[1].rght), (5, 6))
+        self.assertEqual((records[2].lft, records[2].rght), (7, 8))
+        games.refresh_from_db()
+        self.assertEqual((games.lft, games.rght), (3, 10))
+
+    def test_bulk_left(self):
+        games = Category.objects.get(id=3)
+        records = Category.objects.build_tree_nodes(self.games, target=games, position='left')
+        self.assertEqual(len(records), 3)
+        for record in records:
+            self.assertEqual(record.tree_id, games.tree_id)
+        self.assertEqual((records[0].lft, records[0].rght), (3, 8))
+        self.assertEqual((records[1].lft, records[1].rght), (4, 5))
+        self.assertEqual((records[2].lft, records[2].rght), (6, 7))
+        games.refresh_from_db()
+        self.assertEqual((games.lft, games.rght), (9, 10))
